#--------------------------------------#
# 1. previous setting + Y as covariate #
# 2. use mvnfast package               #
#--------------------------------------#

TwoCluster<-function(longdata=NULL,outcomedata=NULL, nknot=4, mcsim, nburnin,ncluster=2, p.null, p.target,ci.true, interim=TRUE,
                    stop.input=NULL){
  library(mvnfast)     # to use rmvn and dmvn function to draw samples
  library(HI)       # to use arms function
  library(gtools)   # to use rdirichlet function
  
  #-------------------------------------#
  # A. Longitudinal part data organize  #
  #-------------------------------------#
  data<-longdata
  
  #---------------------------------------------#
  # I. Spline part creation                     #
  #---------------------------------------------#
  spline.t1<-data[,3]
  spline.t2<-spline.t1^2
  
  pre.k<-seq(0,1,length.out=nknot+2)
  myknot<-pre.k[2:(length(pre.k)-1)]
  spline.f<-function(x,knot){
    myf<-NULL
    for (i in 1:length(knot)){
      myf[i]<-(ifelse(x>=knot[i],(x-knot[i])^2,0))
    }
    return(myf)
  }
  
  spline.t3<-t(sapply(spline.t1,spline.f,knot=myknot))  #dimension: nrow=ngroup*npatient*length(t), ncol=nknot
  z<-as.numeric(data[,1]) # dependent variable, vector format
  
  
  #-------------------------------------------------------------#
  # II. extract dimension parameters from the data:             #
  # number of observations of each patient (N_obs);             #
  # number of total servations(N_tot)= N_subj*length(t);        #
  # number of subjects in the data (N_subj);                    #
  # number of groups in the data (N_grp);                       #
  #-------------------------------------------------------------#
  N_obs<-as.numeric(as.vector(table(data[,2])))
  N_tot<-sum(N_obs)
  N_subj<-as.numeric(length(unique(data[,2])))
  N_grp<-as.numeric(length(unique(data[,4])))
  
  sub<-as.numeric(match(data[,2], unique(data[,2]))) # subject ID indicator
  grp<-as.numeric(match(data[,4], unique(data[,4]))) # group ID indicator
  #------------------------------------------------#
  # III. Vectors to store the posterior samples    #
  # of parameters and initialize them:             #
  # alpha: all the spline coefficients             #
  # beta: all the r.e., ui and wij                 #
  # betaY: the coefficient for X_y                 #
  # tau_u: precision of u_i, group-specific r.e.   #
  # tau_w: precision of w_ij, subject-specfic r.e. #
  # tau_e: precision of e_ijt, measurement error   #
  # tau_phi: precision of phi_k, r.e. of spline    #
  #------------------------------------------------#
  
  alpha<-array(NA, dim=c(mcsim, (3+nknot),ncluster))
  beta<-matrix(NA, nrow=mcsim, ncol=N_grp+N_subj)
  betaY<-rep(NA,mcsim)
  tau_u<-tau_w<-tau_e<-rep(NA,mcsim)
  tau_phi<-matrix(NA, nrow=mcsim, ncol=ncluster)
  alpha.cluster<-c(6, 12) # in an order of ineffective, effective
  # Initialize
  alpha[1, ,]<- rep(0, (3+nknot))
  #alpha[1,1,]<-alpha.cluster
  betaY[1]<-0
  beta[1,]<-rep(0,N_grp+N_subj)
  tau_u[1]<-tau_w[1]<-tau_e[1]<-1
  tau_phi[1,]<-rep(1,ncluster)
  
  
  lowerA0<-c(-50, 10)
  upperA0<-c(10, 50) # in an order of ineffective, effective
  
  #Design matrix for longitudinal part
  X_spline <- cbind(rep(1, N_tot), spline.t1, spline.t2, spline.t3)
  X_u <-NULL
  for (i in 1: N_grp){
    grp.temp<-grp
    grp.temp[which(grp==i)]<-1
    grp.temp[which(grp!=i)]<-0
    X_u<-cbind(X_u,grp.temp)
  }
  X_w<-matrix(0, nrow=N_tot,ncol=N_subj)
  for (i in 1:N_subj){
    X_w[which(sub==i),i]<-1
  }
  X<-cbind(X_spline, X_u, X_w)
  X_random<-cbind(X_u, X_w)
  #------------------------------------------------------------------#
  #-------------------------------------#
  # B. binary part data organize        #
  #-------------------------------------#  
  ydata<-outcomedata
  y<-as.numeric(ydata[,1]) # outcome variable
  
  #----------------------------------------------#
  # I. extract dimension parameters from data    #
  # N_pt: number of patients per group; vector   #
  # N_grp_y: number of groups in outcome data    #
  # N_tot_y: number of total obs in outcome data #
  # X_y: the vector of Y as covariate            #
  #----------------------------------------------#
  N_pt<-as.numeric(as.vector(table(ydata[,2])))
  N_grp_y<-as.numeric(length(unique(ydata[,2])))  
  N_tot_y<-sum(N_pt)
  
  grp_y<-as.numeric(match(ydata[,2], unique(ydata[,2])))# group ID indicator in outcome data
  
  X_y<-NULL
  for (i in 1:N_grp_y){
    X_y<-c(X_y, rep(y[grp_y==i],each=N_obs[1]))
  }
  #------------------------------------------------------#
  # II. Vectors to store the posterior samples           #
  # of parameters and initialize them:                   #
  # mi: the group-specific r.e.                          #
  # theta: the cluster-specific f.e.                     # 
  # tau_mi: precision of m_i(k)                          #
  # p.posterior: posterior of response rate by subgroup  #
  #------------------------------------------------------#
  theta<-matrix(NA, nrow=mcsim, ncol=ncluster)
  tau_mi<-matrix(NA, nrow=mcsim, ncol=ncluster)
  mi<-matrix(NA, nrow=mcsim, ncol=N_grp_y)
  p.posterior<-matrix(NA, nrow=mcsim, ncol=N_grp_y)
  
  mi[1,]<-rep(0, N_grp_y)
  tau_mi[1,]<-rep(2, ncluster)
  
  
  
  theta.p0<-log(p.null/(1-p.null))
  theta.p1<-log(p.target/(1-p.target))
  theta.cluster<-c(theta.p0, theta.p1)
  
  theta[1,]<-theta.cluster
  
  
  lowerB<-c(-5, -1) # in an order of ineffective, effective
  upperB<-c(-1,5)
  
  p.posterior[1,]<-exp(theta.cluster+mi[1,])/(1+exp(theta.cluster+mi[1,]))
  #------------------------------------------------------------------#  
  #----------------------------------------------#
  # C. latent indicator part                     #
  # ci: latent indicator for each subgroup       #
  # pi: Pr(c_ik=1), group i belongs to cluster k #
  # v.prob: allocation probability               #
  #----------------------------------------------#    
  pi.prior<-rep(2,ncluster) # prior specification, use 2 instead of 1/ncluster to avoid sampled p close to 0
  pi<-array(dim=c(mcsim,ncluster,N_grp))
  v.prob<-array(dim=c(mcsim,N_grp,ncluster))
  ci<-array(dim=c(mcsim,ncluster,N_grp))
 
  for (i in 1:N_grp){
    pi[1, ,i]<-c(1-mean(z[grp==i]>=10),mean(z[grp==i]>=10) )
    ci[1, ,i]<-ci.true[i,]
    v.prob[1,i,]<-rep(1/ncluster, ncluster)
  }
    
  
  
  
  
  
  #------------------------------------------------------------------#  
  #----------------------------------------------#
  # D. MCMC part                                 #
  #                                              #
  #----------------------------------------------#    
  a<-0.001; b<-0.001
  a.spline<-10^(-3); b.spline<-10^(-3)
  a.mi<-0.001; b.mi<-0.001
  #indicator.subgrp<-matrix(NA,nrow=(mcsim),ncol=N_grp)
  theta.subgrp<-matrix(NA,nrow=(mcsim), ncol=N_grp)
  tau_mi.subgrp<-matrix(NA,nrow=(mcsim), ncol=N_grp)
  alpha.subgrp<-array(dim=c((mcsim), (3+nknot), N_grp))
  tau_phi.subgrp<-matrix(NA,nrow=(mcsim),ncol=N_grp)
  pi.subgrp<-matrix(NA,nrow=(mcsim), ncol=N_grp)
  
  
  
  #DIC calculation
  
  
  
  
  logbinom<-numeric(N_grp_y)  
  loglongden<-numeric(N_grp) 
  
  logbinom.dev<-numeric(N_grp_y)
  loglongden.dev<-numeric(N_grp)
  
  loglike<-NULL
  #-----------------------------------------------------------#
  # log density of theta and mi in the outcome process,       #
  # will be used in arms                                      #
  #-----------------------------------------------------------#
  
  # log density of theta 
  logden.theta <- function(x, myy, mymi, theta.true) {
    temp <- x + mymi
    return(sum(myy*temp - log(1+exp(temp))) - (10^(-4))*0.5*(x-theta.true)^2)
  }
  
  
  
  # log density of random intercept mi
  # x: the random intercept mi
  logden.mi <- function(x, mytheta, myy, mytau_mi) {
    temp <- mytheta + x
    return(sum(myy * temp - log(1+exp(temp))) - 0.5*(x^2)*mytau_mi)
  }
  
  #log density of tau
  logden.tau<- function(x,mymi, a.mi, b.mi,ngrp){
    return((1+a.mi+ngrp/2)*log(x)-x*(b.mi+0.5*sum(mymi^2)))
  }
  #---------------------------------#
  
  set.seed(100+q)
  for (iter in 2:mcsim){
    #--------------#  
    # update ci    #
    #--------------#  
    for (i in 1:N_grp){
      
      idx.temp<-c(i, (sum(N_pt[1:i])+N_grp-(N_pt[i]-1)):(N_grp+sum(N_pt[1:i]))) # newly added
      sig.long <- (tau_e[iter-1]^(-1)*diag(nrow=N_pt[i]*N_obs[1], ncol=N_pt[i]*N_obs[1]))
      dens.long<-numeric(ncluster)
      dens.binom<-numeric(ncluster)
      for (k in 1:ncluster){
        mean.long <- X_spline[grp==i,]%*%alpha[iter-1, ,k]+X_random[grp==i,idx.temp]%*%beta[iter-1,idx.temp] #updated
        dens.long[k]<-dmvn(z[grp==i], mean.long, sig.long, log=TRUE) #always zero, so take log
        prob.res<- exp(theta[(iter-1),k]+mi[iter-1, i])/(1+ exp(theta[(iter-1),k]+mi[iter-1, i]))
        dens.binom[k]<-dbinom(sum(y[grp_y==i]), N_pt[i], prob.res)
        
      }
      if ((dens.long[1]-dens.long[2])>10^2){
        v.prob[iter,i,]<-c(1,0)
      } else if ((dens.long[1]-dens.long[2])< -10^2) {
        v.prob[iter,i,]<-c(0,1)
      } else {
        num<-(pi[iter-1,1,i]*exp(dens.long[1]-dens.long[2])*dens.binom[1])
        v.prob[iter,i,]<-c(num/(num+pi[iter-1,2,i]*dens.binom[2]),1-num/(num+pi[iter-1,2,i]*dens.binom[2]))
      }
      
      ci[iter,,i]<-as.vector(rmultinom(1, 1, prob=v.prob[iter,i,]))
      
      #-------------# 
      # update pi   #
      #-------------#
      
      pi[iter, ,i] <-sort(rdirichlet(1, (ci[iter, ,i]+pi.prior)), decreasing=FALSE)
    }
    
    #----------------------#  
    # update outcome part  #
    #----------------------#
    
    # update mi
    #update p.posterior
    for (i in 1:N_grp_y){
      temp.myy<-y[grp_y==i]
      k.temp<-which(ci[iter, ,i]==1)
      temp.theta<-theta[iter-1,k.temp]
      temp.tau_mi<-tau_mi[iter-1,k.temp]
      mi[iter, i] <- arms(mi[iter-1, i], logden.mi, function(x,...) (x>(-5))*(x<5), 1,
                          mytheta=temp.theta, myy=temp.myy, mytau_mi=temp.tau_mi)
      p.posterior[iter,i]<-exp(theta[iter-1, k.temp]+mi[iter,i])/(1+exp(theta[iter-1, k.temp]+mi[iter,i]))
    }
    
    # update theta
    # update tau_mi
  
      
    for (k in 1:ncluster){
      clust.temp<-which(ci[iter,k ,]==1) #subgroup indicator of those belonging to cluster k
      sigma.spline<-diag(c(rep(10^4, 3), rep(1/tau_phi[iter-1,k], nknot)), nrow=3+nknot, ncol=3+nknot)
      if (length(clust.temp)==0){
        mean.alpha<-matrix(rep(0,3+nknot),nrow=3+nknot,ncol=1)
        sigma.alpha<-solve(tau_e[iter-1]*(diag(3+nknot))+solve(sigma.spline))
        temp.myy<-mi.long<-NULL
        tau_mi[iter, k] <-rgamma(1, a.mi+length(clust.temp)/2, scale=(b.mi)^(-1) )

      } else {
        
        mi.temp<-mi[iter,clust.temp]
        tau_mi[iter, k] <-rgamma(1, a.mi+length(clust.temp)/2, scale=(b.mi+0.5*sum(mi.temp^2))^(-1) )
        
        temp.myy<-NULL
        myxspline<-NULL
        myxrandom<-NULL
        myz<-NULL
        myxy<-NULL # the X_y vector that belong to cluster k
        for (j in 1:length(clust.temp)){
          temp.myy<-c(temp.myy, y[grp_y==clust.temp[j]])
          myxspline<-rbind(myxspline, X_spline[grp==clust.temp[j],])    #longitudinal part
          myxrandom<-rbind(myxrandom, X_random[grp==clust.temp[j],])    #longitudinal part
          myz<-c(myz, z[grp==clust.temp[j]])                            #longitudinal part
          myxy<-c(myxy, X_y[grp==clust.temp[j]])                        #longitudinal part
        }
        
        mi.long<-rep(mi.temp,N_pt[clust.temp] )
        
    
    #---------------------------#  
    # update longitudinal part  #
    #---------------------------#   
      
      
      
        
        sigma.alpha<-solve(tau_e[iter-1]*crossprod(myxspline)+solve(sigma.spline))
        mean.alpha<-tau_e[iter-1]*sigma.alpha%*%crossprod(myxspline,(myz-myxrandom%*%matrix(beta[iter-1,],ncol=1)-myxy*betaY[iter-1]))
        
        
      } #end else
      
      # update theta
      theta[iter,k]<- arms(theta[iter-1, k], logden.theta, function(x,...) (x>(lowerB[k]))*(x<(upperB[k])), 1, myy=temp.myy, mymi=mi.long, theta.true=theta.cluster[k])
      # update alpha
      alpha[iter, ,k] <-rmvn(1, mean.alpha, sigma.alpha)
      
      
      # update tau_phi #
      
      alpha_spline.temp<-alpha[iter, (4:(3+nknot)), k]
      tau_phi[iter, k]<-rgamma(1, a.spline+0.5*nknot, scale=(b.spline+0.5*sum(alpha_spline.temp^2))^(-1))
      
     
    }# end k

    # update beta
    # update tau_e  #
    res_sum<-0
    betay_vec<-NULL
    for (i in 1: N_grp){
      sigma.random<-diag(c(1/tau_u[iter-1], rep(1/tau_w[iter-1], N_pt[i])), nrow=1+N_pt[i], ncol=1+N_pt[i])
      idx.temp<-c(i, (sum(N_pt[1:i])+N_grp-(N_pt[i]-1)):(N_grp+sum(N_pt[1:i])))
      sigma.beta<-solve(solve(sigma.random)+crossprod(X_random[grp==i,idx.temp])*tau_e[iter-1])
      
      k.temp<-which(ci[iter, ,i]==1)
      mu.beta<-tau_e[iter-1]*sigma.beta%*%crossprod((X_random[grp==i,idx.temp]),(z[grp==i]-X_spline[grp==i,]%*%alpha[iter, ,k.temp]-X_y[grp==i]*betaY[iter-1]))
      
      beta[iter,idx.temp]<-rmvn(1, mu.beta, sigma.beta)
      
      
      res_temp<-z[grp==i]-X_spline[grp==i,]%*%alpha[iter, ,k.temp]-X_random[grp==i,idx.temp]%*%beta[iter,idx.temp]-X_y[grp==i]*betaY[iter-1]
      res_sum<-res_sum+sum(res_temp^2)
      
      betay_temp<-z[grp==i]-X_spline[grp==i,]%*%alpha[iter, ,k.temp]-X_random[grp==i,idx.temp]%*%beta[iter,idx.temp]
      betay_vec<-c(betay_vec, betay_temp)
      # re-organize results 
      # store the membership indicator for each subgroup
      #indicator.subgrp[(iter), i]<-which(ci[iter, ,i]==1)
      
      pi.subgrp[(iter),i]<-pi[iter,k.temp,i]
      theta.subgrp[(iter),i]<-theta[iter,k.temp]
      tau_mi.subgrp[(iter),i]<-tau_mi[iter, k.temp]
      tau_phi.subgrp[(iter),i]<-tau_phi[iter,k.temp]
      alpha.subgrp[(iter),,i]<-alpha[iter, , k.temp]
      
    }
    tau_e[iter]<-rgamma(1, a+0.5*N_tot, scale=(b+0.5*res_sum)^(-1))
    
    # update betaY#
    var.betaY<-1/(10^(-4)+tau_e[iter]*crossprod(X_y))
    mean.betaY<-var.betaY*tau_e[iter]*crossprod(X_y, betay_vec)
    betaY[iter]<-rnorm(1,mean.betaY, sqrt(var.betaY))
    
    # update tau_u#
    tau_u[iter]<-rgamma(1, a+0.5*N_grp, scale=(b+0.5*sum(beta[iter, (1:N_grp)]^2))^(-1))
    
    
    # update tau_w #
    tau_w[iter]<-rgamma(1, a+0.5*N_subj, scale=(b+0.5*sum(beta[iter, ((N_grp+1):(N_grp+N_subj))]^2))^(-1))
    
    
    #Posterior Deviance mean calculation 
    if (iter >=(nburnin+1)){
      for (i in 1:N_grp){
        # loglikelihood for DIC calculation #
        k.temp<-which(ci[iter, ,i]==1)
        idx.temp<-c(i, (sum(N_pt[1:i])+N_grp-(N_pt[i]-1)):(N_grp+sum(N_pt[1:i])))
        
        
        
        
        prob.res<- exp(theta[iter,k.temp]+mi[iter, i])/(1+ exp(theta[iter,k.temp]+mi[iter, i]))
        mi.var<-(tau_mi.subgrp[iter,i])^(-1)
        logbinom[i]<-dbinom(sum(y[grp_y==i]),N_pt[i], prob.res, log=TRUE)
        
        sig.long <- (tau_e[iter]^(-1)*diag(nrow=N_pt[i]*N_obs[1], ncol=N_pt[i]*N_obs[1]))
        mean.long <- X_spline[grp==i,]%*%alpha[iter, ,k.temp]+X_random[grp==i,idx.temp]%*%beta[iter,idx.temp]+X_y[grp==i]*betaY[iter]
        loglongden[i]<-dmvn(z[grp==i], mean.long, sig.long, log=TRUE)
        
      }
      loglike<- c(loglike, -2*sum(loglongden+logbinom))
      
    }
    
    
  }#end MCMC
  
  
  #----------------------------------------------#
  # E. Label switching                           #
  #                                              #
  #----------------------------------------------#   
  
  
  #----------------------------------------------#
  # F. DIC part                                  #
  # loglike: to store the loglikelihood          #
  #----------------------------------------------#   
  
  
  
  #------------------------#  
  
  
  
  dev.bar<-mean(loglike)
  
  # Deviance with posterior mean of parameters
 
  mi.mean<-apply(mi[(nburnin+1):mcsim,],2, mean)
  tau_e.mean<-mean(tau_e[(nburnin+1):mcsim])
  beta.mean<-apply(beta[(nburnin+1):mcsim,],2, mean)
  tau_u.mean<-mean(tau_u[(nburnin+1):mcsim])
  tau_w.mean<-mean(tau_w[(nburnin+1):mcsim])
  pi.mean<-apply(pi.subgrp[(nburnin+1):mcsim,],2, mean)
  theta.mean<-apply(theta.subgrp[(nburnin+1):mcsim,],2, mean)
  alpha.update<-alpha.subgrp[(nburnin+1):mcsim, ,]
  betaY.est<-mean(betaY[(nburnin+1):mcsim])
  # Deviance with posterior mean of parameters
  
 
  
  
  
  
  
  for (i in 1:N_grp){
    prob.postmean<-exp(theta.mean[i]+mi.mean[i])/(1+exp(theta.mean[i]+mi.mean[i]))
    logbinom.dev[i]<-dbinom(sum(y[grp_y==i]),N_pt[i], prob.postmean, log=TRUE)
    
    idx.temp<-c(i, (sum(N_pt[1:i])+N_grp-(N_pt[i]-1)):(N_grp+sum(N_pt[1:i])))
    sig.longmean <- (tau_e.mean^(-1)*diag(nrow=N_pt[i]*N_obs[1], ncol=N_pt[i]*N_obs[1]))
    mu.longmean <- X_spline[grp==i,]%*%apply(alpha.update[, ,i], 2, mean)+X_random[grp==i,idx.temp]%*%beta.mean[idx.temp]+X_y[grp==i]*betaY.est
    loglongden.dev[i]<-dmvn(z[grp==i], mu.longmean, sig.longmean, log=TRUE)
  }
  dev<--2*sum(logbinom.dev+loglongden.dev)

  dic<-2*(dev.bar-dev)+dev
  
  
  #--------------------------------#
  # G. transform the alpha         #
  #--------------------------------#   
  alpha.post<-cbind(alpha[(nburnin+1):mcsim,,1], alpha[(nburnin+1):mcsim,,2])
  #alpha.ave<-colMeans(cbind(alpha[(nburnin+1):mcsim,,1], alpha[(nburnin+1):mcsim,,2]))
  theta.Kmean<-apply(theta[((nburnin+1):mcsim), ], 2, mean)
  ci.save<-ci[((nburnin+1):mcsim),,]
  v.prob.save<-v.prob[((nburnin+1):mcsim),,]
  tau_mi.save<-tau_mi[((nburnin+1):mcsim),]
  
  #--------------------------------#
  # H.Interim/Final analysis       #
  #--------------------------------#  
  futstop<-0.05
  eff.ref<-c(0.911,0.911,0.91,0.909,0.909, 0.909) 
  instruction<-"continue"
  eff.prob<-numeric(N_grp)
  p.post.mean<-numeric(N_grp)
  if (interim == TRUE){
    
    fut.prob<-sapply(seq(1,N_grp_y,1), function(x) mean(p.posterior[((nburnin+1):mcsim),x]>(p.null+p.target)/2))   # calculate the futility probability
    stop.input[which(fut.prob<futstop)]<-1                                                       # stop if the probability <0.05
    
    if (!(0 %in% stop.input)){                                  #if at this interim analysis, all group have stopped for futility
      eff.prob<-sapply(seq(1,N_grp_y,1), function(x) mean(p.posterior[((nburnin+1):mcsim),x]>p.null))
      trteff.indicator<-(eff.prob>eff.ref)*1                                                # summarize the efficacy arms
      p.post.mean<-apply(p.posterior[(nburnin+1):mcsim,],2,mean)
      instruction<-"interim-allstop"
      
    }
    
  } else {  # Final analysis
    
    eff.prob<-sapply(seq(1,N_grp_y,1), function(x) mean(p.posterior[((nburnin+1):mcsim),x]>p.null))
    trteff.indicator<-(eff.prob>eff.ref)*1                                                # summarize the efficacy arms
    p.post.mean<-apply(p.posterior[(nburnin+1):mcsim,],2,mean)
    instruction<-"end"
    
  }
  
  results<-list(trteff.prob=eff.prob, outcome.p.mean = p.post.mean, DIC=dic, 
                instruction=instruction, stop.output=stop.input, v.prob=v.prob.save, ci=ci.save, alpha.post=colMeans(alpha.post),betaY=betaY.est,theta.Kmean=theta.Kmean,tau_mi=tau_mi.save)
  results
  
}#end function

